require 'hashie'
require 'openssl'
require 'yaml'

module ASM
  class Config
    def initialize(options = {})
      config_file = options.delete(:config_file)
      if config_file || options.empty?
        config_file ||= ENV["ASM_CONFIG"] ||
            File::join(File::dirname(__FILE__), '..', '..', 'config.yaml')
        options = YAML.load_file(config_file)
      end
      @mash = Hashie::Mash.new(options)

      [:large_process_concurrency, :large_process_max_runtime,
       :metrics_poller_concurrency, :metrics_poller_timeout].each do |key|
        # These are expected to be integers, but allow code to be specified as a string
        if @mash[key].is_a?(String)
          @mash[key] = eval(@mash[key])
        end
      end

      # Prep http_client_options for use by RestClient
      options = @mash[:http_client_options] ||= {}
      @mash[:http_client_options] = options.to_hash.dup
      [[:ssl_client_cert, OpenSSL::X509::Certificate],
       [:ssl_client_key, OpenSSL::PKey::RSA]].each do |key, init_class|
        if options[key].is_a?(String) && File.exists?(options[key])
          options[key] = init_class.new(File.read(options[key]))
        end
      end
      @mash[:rest_client_options] = options

      # url is expected to be a hash of key => url, make sure it exists
      @mash.url = {} unless @mash.url
    end

    def http_client_options
      @mash[:http_client_options].to_hash
    end

    def rest_client_options
      @mash[:rest_client_options].to_hash.inject({}) { |h, (k, v)| h[k.to_sym] = v; h }
    end

    # Forward methods we don't define directly to the mash
    def method_missing(sym, *args, &block)
      @mash.send(sym, *args, &block)
    end

  end
end