require 'asm/util'
require 'asm/cipher'

module ASM
  class Provider
    class Volume
      # Evaluate if we need same or separate classes for iSCSi, FC and FCoE implementation
      class Compellent < Provider::Base
        puppet_type "asm::volume::compellent"

        property  :size,            :default => "100g",             :validation => /^\d+(k|m|t|KB|MB|GB|TB)$/i
        property  :boot,            :default => "false",            :validation => :boolean
        property  :volumefolder,    :default => "",                 :validation => String
        property  :purge,           :default => "yes",              :validation => ["yes", "no"]
        property  :volume_notes,    :default => "",                 :validation => String
        property  :server_notes,    :default => "",                 :validation => String
        property  :replayprofile,   :default => "Sample",           :validation => String
        property  :storageprofile,  :default => "Low Priority",     :validation => String
        property  :servername,      :default => "",                 :validation => String
        property  :operatingsystem, :default => "VMWare ESX 5.1",   :validation => String
        property  :serverfolder,    :default => "",                 :validation => String
        property  :wwn,             :default => "",                 :validation => String
        property  :porttype,        :default => "FibreChannel",     :validation => String
        property  :manual,          :default => "false",            :validation => :boolean
        property  :force,           :default => "false",            :validation => :boolean
        property  :readonly,        :default => "false",            :validation => :boolean
        property  :singlepath,      :default => "false",            :validation => :boolean
        property  :lun,             :default => "",                 :validation => String
        property  :localport,       :default => "",                 :validation => String
        property  :ensure,          :default => "present",          :validation => ["present", "absent"]

        # @todo the old code short circuited the thing and only proccessed
        #       the first server object from the first related volume, this might
        #       be fine but I dont know how we'll get the same short circuit in here
        #       as the loop is happening in the Provider::Server#clean_related_volumes!.
        #       Perhaps the remove_server_from_volume! api could consider return value
        #       to mean something to indicate the short circuit
        def remove_server_from_volume!(server, options={})
          server_object = compellent_server_resource(server)
          type.process_generic(type.puppet_certname, server_object, "apply", true, nil, type.guid)
        end

        def clean_access_rights!
          resources = {}

          type.related_servers.select{|server| server.teardown?}.each do |server|
            logger.warn("Cleaning access rights for server %s from volume %s" % [server.puppet_certname, type.puppet_certname])

            resource = compellent_server_resource(server)

            resources["compellent_server"] ||= {}
            resources["compellent_server"].merge!(resource["compellent_server"])
          end

          unless resources.empty?
            type.process_generic(type.puppet_certname, resources, "apply", true, nil, type.guid)
          end
        end

        def esx_datastore(server, cluster, volume_ensure)
          datastore_name = "%s:%s" % [server.hostip, @uuid]

          if !debug?
            device_id = ASM::Util.find_compellent_volume_info(type.guid, @uuid, self["volumefolder"], logger)
            decrypt_password = ASM::Cipher.decrypt_string(server.admin_password)
            lun_id = type.deployment.get_compellent_lunid(server.hostip, 'root', decrypt_password, device_id)
          else
            device_id = "DEBUG_DEVICE_ID"
            decrypt_password = "DEBUG_PASSWORD"
            luni_id = "DEBUG_LUNID"
          end

          {"esx_datastore" =>
            {datastore_name =>
              {"ensure" => volume_ensure,
               "datastore" => @uuid,
               "type" => "vmfs",
               "lun" => lun_id,
               "transport" => "Transport[vcenter]"
              }
            }
          }
        end

        def compellent_server_resource(server)
          server_object_name = "ASM_%s" % server.cert2serial

          server_object = {"compellent_server" => {
            server_object_name => {
              "ensure"       => "absent",
              "serverfolder" => ""
            }
          }}
        end
      end
    end
  end
end

