module ASM
  class Provider
    class Volume
      # Provider that is capable of handling basic creation and destruction of Equallogic
      # volumes based on the +asm::volume::equallogic+ puppet class
      class Netapp < Provider::Base
        puppet_type "netapp::create_nfs_export"
        puppet_run_type "device"

        DEFAULT_SNAP_SCHED = {"minutes" => 0, "hours" => 0, "days" => 0, "weeks" => 0, "which-hours" => 0, "which-minutes" => 0}
        DEFAULT_OPTIONS = {"convert_ucode" => "on", "no_atime_update" => "on", "try_first" => "volume_grow"}

        property  :size,              :default => nil,                :validation => /^\d+(k|g|m|t|KB|GB|MB|TB)$/
        property  :ensure,            :default => "present",          :validation => ["present", "absent"]
        property  :aggr,              :default => "aggr1",            :validation => String
        property  :spaceres,          :default => "none",             :validation => ["none", "file", "volume"]
        property  :snapresv,          :default => 0,                  :validation => String
        property  :autoincrement,     :default => true,               :validation => :boolean
        property  :persistent,        :defautl => true,               :validation => :boolean
        property  :readonly,          :default => "",                 :validation => Object       # string or array :(
        property  :readwrite,         :default => ["all_hosts"],      :validation => Object       # string or array :(
        property  :snapschedule,      :default => DEFAULT_SNAP_SCHED, :validation => Hash
        property  :options,           :default => DEFAULT_OPTIONS,    :validation => Hash

        def configure_hook
          self[:size].gsub!(/KB/,'k')
          self[:size].gsub!(/GB/,'g')
          self[:size].gsub!(/MB/,'m')
          self[:size].gsub!(/TB/,'t')
        end

        def cluster_supported?(cluster)
          cluster.provider_path == "cluster/vmware"
        end

        def clean_access_rights!; end

        def remove_server_from_volume!(server)
          logger.warn("Removing access for server %s from NetApp volume %s has not been implemented" % [server.puppet_certname, type.puppet_certname])
        end

        def esx_datastore(server, cluster, volume_ensure)
          dc_name = cluster.provider.datacenter
          cluster_name = cluster.provider.cluster

          path = "/%s/%s/" % [dc_name, cluster_name]
          datastore_name = "%s:%s" % [server.hostip, @uuid]

          {"esx_datastore" =>
            {datastore_name =>
              {"ensure" => volume_ensure,
               "path" => path,
               "type" => "nfs",
               "transport" => "Transport[vcenter]"
              }
            }
          }
        end
      end
    end
  end
end
