dir = File.expand_path(File.dirname(__FILE__))
$LOAD_PATH.unshift File.join(dir, 'lib')

# Don't want puppet getting the command line arguments for rake or autotest
ARGV.clear

require 'puppet'
require 'facter'
require 'mocha/api'
gem 'rspec', '>=2.0.0'
require 'rspec/expectations'

require 'puppetlabs_spec_helper/module_spec_helper'

module SpecHelper
  FIXTURE_PATH = File.expand_path(File.join(File.dirname(__FILE__), "fixtures"))

  def self.load_fixture(fixture)
    File.read(File.join(FIXTURE_PATH, fixture))
  end

  def self.service_from_fixture(fixture)
    fixture = JSON.parse(load_fixture(fixture), :max_nesting => 100)
    ASM::Service.new(fixture)
  end

  def self.init_i18n
    I18n.load_path = Dir[File.expand_path(File.join(File.dirname(__FILE__), "..", "locales", "*.yml"))]
    I18n.locale = "en".intern
  end
end

module ASM
  # TODO: we should probably use config.yaml "environments" for this like razor does
  def self.test_config_file
    if RUBY_PLATFORM == 'java'
      File.join(File.dirname(__FILE__), 'jruby_config.yaml')
    else
      File.join(File.dirname(__FILE__), 'mri_config.yaml')
    end
  end

  def self.initialized_for_tests
    @initialized_for_tests
  end

  def self.init_for_tests
    require 'asm'
    require 'logger'
    # This is an ugly kludge, need to make the base_directory configurable and
    # just have the logger log to a temporary directory
    logger = Logger.new(nil)
    ASM.stubs(:logger).returns(logger)
    ASM.init(self.test_config_file)
    @initialized_for_tests = true
  end
end

RSpec.configure do |config|
  # FIXME REVISIT - We may want to delegate to Facter like we do in
  # Puppet::PuppetSpecInitializer.initialize_via_testhelper(config) because
  # this behavior is a duplication of the spec_helper in Facter.
  config.before :each do
    # Ensure that we don't accidentally cache facts and environment between
    # test cases.  This requires each example group to explicitly load the
    # facts being exercised with something like
    # Facter.collection.loader.load(:ipaddress)
    Facter::Util::Loader.any_instance.stubs(:load_all)
    Facter.clear
    Facter.clear_messages
  end
end
