namespace :deployment do
  desc "View the contents of a deployment JSON set with DEPLOYMENT=deployment.json"
  task :inspect do
    require 'asm/service'
    require 'json'
    require 'logger'

    file = File.expand_path(ENV["DEPLOYMENT"] || "deployment.json")

    service = ASM::Service.new(JSON.parse(File.read(file), :max_nesting => 100))

    puts "Deployment file %s" % file
    puts
    puts "    Deployment Name: %s" % service.deployment_name
    puts "                 ID: %s" % service.id
    puts "           Teardown: %s" % service.teardown?
    puts "              Retry: %s" % service.retry?
    puts "          Migration: %s" % service.migration?
    puts
    puts "Components:"

    service.components.each do |component|
      puts
      puts "    %s" % component
      puts "           Name: %s" % component.name
      puts "           GUID: %s" % component.guid
      puts "       Teardown: %s" % component.teardown?
      puts "           Type: %s" % component.type
      puts "             ID: %s" % component.component_id
      puts "      Cert Name: %s" % component.puppet_certname
      puts "      Resources: %s" % component.resource_ids.inspect
      begin
        puts "       Provider: %s" % component.to_resource(nil, Logger.new(nil)).provider_path
      rescue LoadError
        puts "       Provider: unknown component type"
      end
    end

    puts

  end
end
